package ri25c3.ctf;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import net.sourceforge.adela.exceptions.ADELASyntaxException;
import net.sourceforge.adela.interpreter.ADELAFunctionBlock;
import net.sourceforge.adela.interpreter.ADELAParser;

public final class ADELACache {

	public class FInfo {
		private final long lastModified;

		private final Map<String, ADELAFunctionBlock> code;

		public FInfo(long lm, Map<String, ADELAFunctionBlock> code) {
			this.lastModified = lm;
			this.code = code;
		}

		public Map<String, ADELAFunctionBlock> getCode() {
			return code;
		}

		public long getLastModified() {
			return lastModified;
		}

		@Override
		public int hashCode() {
			final int PRIME = 31;
			int result = 1;
			result = PRIME * result + ((code == null) ? 0 : code.hashCode());
			result = PRIME * result
					+ (int) (lastModified ^ (lastModified >>> 32));
			return result;
		}

		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			final FInfo other = (FInfo) obj;
			if (code == null) {
				if (other.code != null)
					return false;
			} else if (!code.equals(other.code))
				return false;
			if (lastModified != other.lastModified)
				return false;
			return true;
		}
	}

	private static final ADELACache instance = new ADELACache();

	private Map<String, FInfo> files = new HashMap<String, FInfo>();

	private ADELACache() {
	}

	private Map<String, ADELAFunctionBlock> parse(File code)
			throws IOException, ADELASyntaxException {
		FileReader fr = new FileReader(code);
		char[] buf = new char[(int) code.length()];
		int len = fr.read(buf);
		String s = new String(buf, 0, len);
		return new ADELAParser().BuildAutomat(s);
	}

	public synchronized ADELAFunctionBlock getFunction(String path,
			String function) throws IOException, ADELASyntaxException {
		FInfo info = files.get(path);
		if (info == null) {
			info = new FInfo(new File(path).lastModified(),
					parse(new File(path)));
			files.put(path, info);
		} else {
			if (info.getLastModified() != new File(path).lastModified()) {
				info = new FInfo(new File(path).lastModified(), parse(new File(
						path)));
				files.put(path, info);
			}
		}
		return info.getCode().get(function);
	}

	public static ADELACache getInstance() {
		return instance;
	}
}
