package ri25c3.ctf;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

public final class URLCache {
	private static URLCache uc = new URLCache();

	private Map<String, Assignment[]> patterns = new HashMap<String, Assignment[]>();

	private Map<String, Long> lastloaded = new HashMap<String, Long>();

	public static URLCache getInstance() {
		return uc;
	}

	public synchronized AM getAssignment(String user, String path)
			throws IOException, URLSException {
		Long lastLoaded = lastloaded.get(user);
		File f = new File(String.format("userdata/%s/.urls", user));
		if (!f.exists()) {
			return null;
		}
		Assignment[] assis;
		if ((lastLoaded == null) || (lastLoaded != f.lastModified())) {
			assis = loadAssis(f);
		} else {
			assis = patterns.get(user);
		}
		for (int i = 0; i < assis.length; i++) {
			Matcher m = assis[i].getPattern().matcher(path);
			if (m.matches()) {
				return new AM(assis[i], m);
			}
		}
		return null;
	}

	private Assignment[] loadAssis(File f) throws IOException, URLSException {
		BufferedReader br = new BufferedReader(new FileReader(f));
		ArrayList<Assignment> assis = new ArrayList<Assignment>();
		while (br.ready()) {
			String line = br.readLine();
			if (line.trim().length() == 0) {
				continue;
			}
			String[] parms = line.split("\t");
			if (parms.length != 3) {
				throw new URLSException("Invalid line");
			}
			Pattern p;
			try {
				p = Pattern.compile(parms[0]);
			} catch (PatternSyntaxException pe) {
				throw new URLSException("Invalid regular expression");
			}
			assis.add(new Assignment(p, parms[1], parms[2]));
		}
		return assis.toArray(new Assignment[0]);
	}
}
