package ri25c3.ctf.handlers;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;

import net.sourceforge.adela.exceptions.ADELASyntaxException;
import net.sourceforge.adela.exceptions.FunctionNotFoundException;
import net.sourceforge.adela.exceptions.WrongParameterCountException;
import net.sourceforge.adela.interpreter.ADELAException;
import net.sourceforge.adela.interpreter.ADELAFunctionBlock;
import net.sourceforge.adela.interpreter.ADELAInterpreter;

import ri25c3.ctf.ADELACache;
import ri25c3.ctf.AM;
import ri25c3.ctf.Assignment;
import ri25c3.ctf.RequestHandler;
import ri25c3.ctf.Sessions;
import ri25c3.ctf.URLCache;
import ri25c3.ctf.URLSException;

public class Get implements RequestHandler {

	public boolean handle(BufferedReader in, PrintStream out, String[] request,
			Map<String, List<String>> parms) {
		// hack
		if ("GET".equals(request[0])) {
			in = null;
		}
		String user;
		String path = request[1];
		List<String> vh = parms.get("host");
		if (vh != null) {
			user = vh.get(0).split("\\.", 2)[0];
		} else {
			String[] foo = path.substring(2).split("\\/", 2);
			user = foo[0];
			path = "/" + foo[1];
		}
		System.out.printf(request[0] + " USER: %s PATH: %s\n", user, path);
		if (user.split("[A-Za-z][A-Za-z0-9]+").length != 0) {
			out.println("HTTP/1.1 404 INVALID USER");
			out
					.println("Content-type: text/html\n\n<html><body><h1>404 Invalid user</h1></body></html>\n");
			return false;
		}

		// first find ADELA content
		try {
			AM am = URLCache.getInstance().getAssignment(user, path);
			if (am != null) {
				Assignment a = am.getA();
				Matcher m = am.getM();
				System.out.printf("MATCH: %s %s %s\n", a.getPattern(), a
						.getPath(), a.getFunction());
				ADELAFunctionBlock afb = ADELACache.getInstance().getFunction(
						String.format("userdata/%s/%s", user, a.getPath()),
						a.getFunction());
				Object[] params = new Object[5 + m.groupCount()];
				params[0] = new HTTPRequest(parms, request);
				params[1] = new HTTPResponse();
				params[2] = new HTTPDatabase(user);
				params[3] = null;
				List<String> sess = parms.get("session");
				if ((sess != null) && (sess.size() > 0)) {
					params[3] = Sessions.getSession(sess.get(0), false);
				}
				if (params[3] == null) {
					System.out.println("WARNING: Creating temporary session");
					params[3] = new HashMap<Object, Object>();
				}
				params[4] = in;
				for (int i = 0; i < m.groupCount(); i++) {
					params[i + 5] = m.group(i + 1);
				}
				ADELAInterpreter evaluator = new ADELAInterpreter();
				evaluator.DeclareClassSafe(HTTPRequest.class);
				evaluator.DeclareClassSafe(HTTPResponse.class);
				evaluator.DeclareClassSafe(HTTPDatabase.class);
				evaluator.DeclareClassSafe(HashMap.class);
				evaluator.DeclareClassSafe(BufferedReader.class);
				evaluator.AllowFullAccess("StringBuilder", StringBuilder.class);
				String response = (String) afb.Handle(evaluator, params);
				out.println("HTTP/1.1 200 OK");
				out.println("Content-type: "
						+ ((HTTPResponse) params[1]).getContentType() + "\n");
				out.println(response);
				return true;
			}
		} catch (IOException e1) {
			out.println("HTTP/1.1 404 IOException\nContent-type: text/plain\n");
			out.println(e1.getMessage());
			return false;
		} catch (URLSException e1) {
			out
					.println("HTTP/1.1 404 URLSException\nContent-type: text/plain\n");
			out.println(e1.getMessage());
			return false;
		} catch (ADELASyntaxException e) {
			out
					.println("HTTP/1.1 404 ADELASyntaxException\nContent-type: text/plain\n");
			out.println(e.getMessage());
			return false;
		} catch (ADELAException e) {
			out
					.println("HTTP/1.1 404 ADELAException\nContent-type: text/plain\n");
			out.println(e.getMessage());
			return false;
		} catch (WrongParameterCountException e) {
			out
					.println("HTTP/1.1 404 WrongParameterCountException\nContent-type: text/plain\n");
			out.println(e.getMessage());
			return false;
		} catch (FunctionNotFoundException e) {
			out
					.println("HTTP/1.1 404 FunctionNotFoundException\nContent-type: text/plain\n");
			out.println(e.getMessage());
			return false;
		}

		if (request[1].endsWith(".html")) {
			// static
			try {
				BufferedReader ibr = new BufferedReader(new FileReader(String
						.format("userdata/%s/%s", user, path)));
				out
						.println("HTTP/1.1 200 OK\nContent-type: text/html; encoding=utf-8\n\n");
				while (ibr.ready()) {
					out.println(ibr.readLine());
				}
				ibr.close();
				return true;
			} catch (FileNotFoundException e) {
				out
						.println("HTTP/1.1 404 NOT FOUND\nContent-type: text/plain\n\n404 NOT FOUND\n");
				return false;
			} catch (IOException e) {
				out
						.println("HTTP/1.1 404 IO ERROR\nContent-type: text/plain\n\n404 IO ERROR\n");
				return false;
			}
		} else if (request[1].endsWith(".cgi")) {
			String[] cmdarray = new String[1];
			cmdarray[0] = String.format("userdata/%s/%s", user, path);
			try {
				Process p = Runtime.getRuntime().exec(cmdarray);
				BufferedReader br = new BufferedReader(new InputStreamReader(p
						.getInputStream()));
				OutputStream pos = p.getOutputStream();
				if (in != null) {
					pos.write(in.read());
					while (in.ready()) {
						char ctr = (char) in.read();
						if (ctr == -1) {
							break;
						}
						pos.write(ctr);
					}
				}
				pos.close();
				out.println("HTTP/1.1 200 OK");
				while (true) {
					String line = br.readLine();
					if (line == null) {
						break;
					}
					out.println(line);
				}
			} catch (IOException e) {
				out
						.println("HTTP/1.1 404 EXEC failed\nContent-type: text/plain\n\n404 EXEC failed\n");
				return false;
			}
		}
		return true;
	}
}
